<?php

/**
 * Class for registering a new settings page under Settings.
 */
class Clonable_Options_Page {

    private $name;
    private $version;

    /**
     * Constructor.
     * @param $name
     * @param $version
     */
    function __construct($name, $version) {
        $this->name = $name;
        $this->version = $version;

        add_action('admin_menu', array($this, 'admin_menu'));
        add_action('admin_init', array($this, 'register_settings'));
    }

    /**
     * Registers a new settings page under Settings.
     */
    function admin_menu() {
        add_options_page(
            __('Clonable Settings', 'textdomain'),
            __('Clonable', 'textdomain'),
            'manage_options',
            'clonable-settings',
            array(
                $this,
                'settings_page'
            )
        );
    }

    function register_settings() {
        add_settings_section('clonable_langtags', 'Language Tag Settings', array($this, 'clonable_langtags_desc'), 'clonable_settings');

        $args = array(
            'sanitize_callback' => array($this, 'clonable_langtag_data_validate'),
            'default' => NULL,
        );
        register_setting('clonable_options', 'clonable_langtag_data', $args);
        add_settings_field('clonable_langtag_data', 'Domain data (JSON)', array($this, 'clonable_langtag_data_field'), 'clonable_settings', 'clonable_langtags');
    }

    function clonable_langtags_desc() {
        echo '<p>Use the list below to manage the language tags on your clone. Language tags help search engines get a
                better view of the multiple versions of your site. You can find the data you need to enter in the
                Clonable Dashboard. For more information, click <a href="https://kb.clonable.net/">here</a>.</p>';
    }

    function clonable_langtag_data_field() {
        $option = get_option('clonable_langtag_data');
        echo "<textarea id='clonable_langtag_data' name='clonable_langtag_data' rows='8' style='min-width: 500px; width:50%; max-width: 100%; font-family: monospace; resize: none'>" . esc_textarea($option) . "</textarea>";
    }

    function clonable_langtag_data_validate( $input ) {
        $json = json_decode($input);
        if ($json === null) {
            add_settings_error('clonable_langtag_data', 'err_invalid_json', 'The supplied JSON string was invalid. Please check its syntax and try again.');
            return null;
        }

        if ($json->version !== 1) {
            add_settings_error('clonable_langtag_data', 'err_invalid_schema_version', 'The supplied JSON schema has the wrong version. Make sure the plug-in is up to date.');
            return null;
        }

        if (!isset($json->data)) {
            add_settings_error('clonable_langtag_data', 'err_no_data', 'The supplied JSON does not contain any data.');
            return null;
        } else {
            $data = $json->data;
        }

        // Validate original if present
        if (isset($data->original)) {
            if (!isset($data->original->domain) || !isset($data->original->langcode)) {
                add_settings_error('clonable_langtag_data', 'err_original_missing_data', 'The original is missing data. Please check if it has a domain name and language code field.');
                return null;
            }

            // Check if valid domain
            if (filter_var($data->original->domain, FILTER_VALIDATE_DOMAIN) === false) {
                add_settings_error('clonable_langtag_data', 'err_original_invalid_domain', 'The original domain is invalid.');
                return null;
            }

            // Check if valid langcode
            if (!preg_match('/^[a-z]{2}-[a-z]{2}$/', $data->original->langcode)) {
                add_settings_error('clonable_langtag_data', 'err_original_invalid_langcode', 'The original language code is invalid.');
                return null;
            }
        }

        // Validate clones
        if (isset($data->clones)) {
            if (!is_array($data->clones)) {
                add_settings_error('clonable_langtag_data', 'err_invalid_clone_array', 'The clones list is invalid.');
                return null;
            }

            foreach ($data->clones as $clone) {
                if (!isset($clone->domain) || !isset($clone->langcode)) {
                    add_settings_error('clonable_langtag_data', 'err_clone_missing_data', 'A clone in the list is missing data. Please check if all clones have a domain name and language code field.');
                    return null;
                }

                // Check if valid domain
                if (filter_var($clone->domain, FILTER_VALIDATE_DOMAIN) === false) {
                    add_settings_error('clonable_langtag_data', 'err_clone_invalid_domain', "The domain name of clone $clone->domain is invalid.");
                    return null;
                }

                // Check if valid langcode
                if (!preg_match('/^[a-z]{2}-[a-z]{2}$/', $clone->langcode)) {
                    add_settings_error('clonable_langtag_data', 'err_clone_invalid_langcode', "The language tag name of clone $clone->domain is invalid.");
                    return null;
                }

                // Check subfolder settings
                $original_subfolder = $clone->original_subfolder;
                $clone_subfolder = $clone->clone_subfolder;
                if (!(c_str_starts_with($original_subfolder, '/') && c_str_ends_with($original_subfolder, '/'))) {
                    add_settings_error('clonable_langtag_data', 'err_invalid_original_subfolder', "The original subfolder of $clone->domain is invalid.");
                    return null;
                }

                if (!(c_str_starts_with($clone_subfolder, '/') && c_str_ends_with($clone_subfolder, '/'))) {
                    add_settings_error('clonable_langtag_data', 'err_invalid_clone_subfolder', "The clone subfolder of $clone->domain is invalid.");
                    return null;
                }
            }
        }

        return $input;
    }

    /**
     * Settings page display callback.
     */
    function settings_page() {
        ?>
        <div>
            <h1>Clonable Settings</h1>
            <p>Here you can configure the settings of the Clonable wordpress integration.</p>
            <form method="post" action="options.php">
                <?php settings_fields('clonable_options');
                do_settings_sections('clonable_settings'); ?>

                <?php submit_button(); ?>
            </form>
        </div>

        <?php
    }
}